% Program : LASSO_risk_hw1.m
% Purpose: Plot the LASSO risk versus the regularization parameter lambda
% Author: Song Mei
% Date: Dec 19, 2020

%% Model parameters

sigma = sqrt(0.2);
delta = 0.64;
p = 0.064;
m =  1;
eps_bisection = 1e-5;

%% Define function handles and the parameter alpha_min

eta_threshold = @(x, t) sign(x) .* (abs(x) > t) .* (abs(x) - t);
d_eta_threshold = @(x, t) (abs(x) > t);
Fixed_alpha = @(alpha, delta) (1 + alpha.^2) .* normcdf(- alpha) - alpha .* normpdf(alpha) - delta/2;
alpha_min = bisection(@(alpha)Fixed_alpha(alpha, delta),0,10,eps_bisection,1);
F_tau_theta = @(tau, theta) sigma^2 + (1 / delta) * integral(...
    @(g) p * (eta_threshold( m + tau * g, theta) - m).^2 .* normpdf(g) ... 
    + p * (eta_threshold(- m + tau * g, theta) + m).^2 .* normpdf(g) ... 
    + (1 - 2 * p) * (eta_threshold(tau * g, theta)).^2 .* normpdf(g), -inf, inf);
Fixed_tau = @(tau, alpha) tau^2 - F_tau_theta(tau, alpha * tau);
tau_alpha = @(alpha) bisection(@(tau) Fixed_tau(tau, alpha),0,20,eps_bisection,1);
lambda_func = @(alpha, tau) alpha * tau * ...
    (1 - (1/ delta) * ...
    integral(...
    @(g) p * d_eta_threshold(m + tau * g, alpha * tau) .* normpdf(g) ... 
    + p * d_eta_threshold(-m + tau * g, alpha * tau) .* normpdf(g) ... 
    + (1 - 2 * p) * d_eta_threshold(tau * g, alpha * tau) .* normpdf(g), -inf, inf));
lambda_func(alpha, tau);
R_func = @(alpha) delta * (tau_alpha(alpha)^2 - sigma^2);

%% Calculate the risk curve

alpha_set = [alpha_min + 0.05: 0.05: 5];

for iter_alpha = 1:length(alpha_set)
    alpha = alpha_set(iter_alpha);
    tau = tau_alpha(alpha);
    tau_res(iter_alpha) = tau;
    lambda_res(iter_alpha) = lambda_func(alpha, tau);
    R_res(iter_alpha) = R_func(alpha);
end

%% Plot the figure

fig = figure(1);
plot(lambda_res, R_res, 'linewidth', 2)
xlim([0, 3])
set(gca, 'fontsize', 14);
xlabel('$\lambda$', 'fontsize', 14, 'interpreter', 'latex')
ylabel('Risk', 'fontsize', 14, 'interpreter', 'latex')
title('$\delta = 0.64$, $\sigma^2 = 0.2$, $P(x = 1) = P(x = -1) = 0.064$', 'fontsize', 14, 'interpreter', 'latex')

% print(fig, 'Lasso_risk', '-depsc2')


