function root = bisection(f,a,b,epsilon,ifmute)
% Program : bisection.m
%
% Purpose : find the root of a given single variable function within the
%           given interval
%
% Inputs :  At least three input arguments are required viz., function, initial point of the
%           interval, and final point of the interval
% Syntax :  root = bisection(f,a,b)
%           where f is an inline function 
%           e.g., f = inline('sin(x)','x');
%           a and b are the end point of the interval
%
% Example:  root = bisection(f,-3,5)
%           The fourth input argument is the stopping criteria
%           If the function value deceeds epsilon, the loop terminates
%           giving the root. Default is 1e-6.
%
% Syntax :  root = bisection(f,a,b,epsilon)
%
% Example:  root = bisection(f,1,2,1e-9)
%           where f is again an inline function as mentioned above

if nargin < 5
    ifmute = 0;
end
    fa = f(a);
    fb = f(b);
    if (fa*fb >= 0)
        disp('The function has positive value at the end points of interval.');
        disp('The root can''t be found using bisection method, use some other method.');
        root = 'Root can''t be found using bisection method';
        return;
    else
        if ~ifmute
        fprintf('Iteration\t\t\ta\t\t\tb\t\t\tm\t\t\tf(a)\t\t\tf(b)\t\t\tf(m)\n');
        fprintf('=========\t\t ======\t\t ======\t\t ======\t\t   ======\t\t   ======\t\t   ======\n');
        end
        m = (a + b)/2;
        fm = f(m);
        if (abs(fm) <= epsilon)
            root = m;
            return;
        else
            Iter = 0;
            %while (abs(f(m)) >= epsilon) & Iter < 100
            while (abs(b-a) >= epsilon) & Iter < 100
                Iter = Iter + 1; 
                m = (a + b)/2;
                fm = f(m);
                if(fa*fm > 0)
                    a = m;
                    fa = fm;
                else
                    b = m;
                    fb = fm;
                end
                if mod(Iter, 1) == 0
                    if ~ifmute
                    fprintf('%3d',Iter);
                    fprintf('%20.4f',a);
                    fprintf('%12.4f',b);
                    fprintf('%12.4f',m);
                    fprintf('%14.4f',fa);
                    fprintf('%16.4f',fb);
                    fprintf('%16.4f',fm);
                    fprintf('\n');
                    end
                end
            end
            root = m;
        end

end